@@ -1,7 +1,7 @@
 package Proc::Govern;
 
-our $DATE = '2016-04-07'; # DATE
-our $VERSION = '0.18'; # VERSION
+our $DATE = '2017-03-31'; # DATE
+our $VERSION = '0.19'; # VERSION
 
 use 5.010001;
 use strict;
@@ -67,8 +67,8 @@ $SPEC{govern_process} = {
     summary => 'Run child process and govern its various aspects',
     description => <<'_',
 
-It basically uses `IPC::Run` and a loop to check various conditions during the
-lifetime of the child process.
+It basically uses <pm:IPC::Run> and a loop to check various conditions during
+the lifetime of the child process.
 
 TODO: restart_delay, check_alive.
 
@@ -79,7 +79,7 @@ _
             description => <<'_',
 
 Should match regex `\A\w+\z`. Used in several places, e.g. passed as `prefix` in
-`File::Write::Rotate`'s constructor as well as used as name of PID file.
+<pm:File::Write::Rotate>'s constructor as well as used as name of PID file.
 
 If not given, will be taken from command.
 
@@ -91,7 +91,7 @@ _
             summary => 'Command to run',
             description => <<'_',
 
-Passed to `IPC::Run`'s `start()`.
+Passed to <pm:IPC::Run>'s `start()`.
 
 _
         },
@@ -100,11 +100,12 @@ _
             description => <<'_',
 
 If set to true, will prevent running multiple instances simultaneously.
-Implemented using `Proc::PID::File`. You will also normally have to set
+Implemented using <pm:Proc::PID::File>. You will also normally have to set
 `pid_dir`, unless your script runs as root, in which case you can use the
 default `/var/run`.
 
 _
+            tags => ['category:instance-control'],
         },
         pid_dir => {
             summary => 'Directory to put PID file in',
@@ -118,6 +119,7 @@ Can be set to `exit` to silently exit when there is already a running instance.
 Otherwise, will print an error message `Program <NAME> already running`.
 
 _
+            tags => ['category:instance-control'],
         },
         load_watch => {
             schema => [bool => default => 0],
@@ -130,30 +132,32 @@ _
         },
         load_check_every => {
             schema => [duration => default => 10],
-            'x.perl.coerce_to' => 'int(secs)',
             summary => 'Frequency of load checking',
+            tags => ['category:load-control'],
         },
         load_high_limit => {
             schema => ['any*' => of => [[int => default => 1.25], 'code*']],
             description => <<'_',
 
 Limit above which program should be suspended, if load watching is enabled. If
-integer, will be compared against `Unix::Uptime->load`'s `$load1` value.
+integer, will be compared against <pm:Unix::Uptime>`->load`'s `$load1` value.
 Alternatively, you can provide a custom routine here, code should return true if
 load is considered too high.
 
 _
+            tags => ['category:load-control'],
         },
         load_low_limit => {
             schema => ['any*' => of => [[int => default => 0.25], 'code*']],
             description => <<'_',
 
 Limit below which program should resume, if load watching is enabled. If
-integer, will be compared against `Unix::Uptime->load`'s `$load1` value.
+integer, will be compared against <pm:Unix::Uptime>`->load`'s `$load1` value.
 Alternatively, you can provide a custom routine here, code should return true if
 load is considered low.
 
 _
+            tags => ['category:load-control'],
         },
         killfam => {
             summary => 'Instead of kill, use killfam (kill family of process)',
@@ -163,7 +167,7 @@ _
 This can be useful e.g. to control load more successfully, if the
 load-generating processes are the subchildren of the one we're governing.
 
-This requires `Proc::Killfam` CPAN module, which is installed separately.
+This requires <pm:Proc::Killfam> CPAN module, which is installed separately.
 
 _
         },
@@ -174,22 +178,24 @@ _
                 size      => 'str*',
                 histories => 'int*',
             }],
+            tags => ['category:logging'],
         },
         show_stdout => {
             schema => [bool => default => 1],
             summary => 'Just like `show_stderr`, but for STDOUT',
+            tags => ['category:output-control'],
         },
         log_stderr => {
             summary => 'Will be passed as arguments to `File::Write::Rotate`',
             description => <<'_',
 
-Specify logging for STDERR. Logging will be done using `File::Write::Rotate`.
+Specify logging for STDERR. Logging will be done using <pm:File::Write::Rotate>.
 Known hash keys: `dir` (STR, defaults to `/var/log`, directory, preferably
 absolute, where the log file(s) will reside, should already exist and be
-writable, will be passed to `File::Write::Rotate`'s constructor), `size` (int,
-also passed to `File::Write::Rotate`'s constructor), `histories` (int, also
-passed to `File::Write::Rotate`'s constructor), `period` (str, also passed to
-`File::Write::Rotate`'s constructor).
+writable, will be passed to <pm:File::Write::Rotate>'s constructor), `size`
+(int, also passed to <pm:File::Write::Rotate>'s constructor), `histories` (int,
+also passed to <pm:File::Write::Rotate>'s constructor), `period` (str, also
+passed to <pm:File::Write::Rotate>'s constructor).
 
 _
             schema => ['hash*' => keys => {
@@ -197,6 +203,7 @@ _
                 size      => 'str*',
                 histories => 'int*',
             }],
+            tags => ['category:logging'],
         },
         show_stderr => {
             schema => ['bool'],
@@ -207,6 +214,7 @@ Can be used to turn off STDERR output. If you turn this off and set
 `log_stderr`, STDERR output will still be logged but not displayed to screen.
 
 _
+            tags => ['category:output-control'],
         },
         timeout => {
             schema => ['int*'],
@@ -217,25 +225,33 @@ After this time is reached, process (and all its descendants) are first sent the
 TERM signal. If after 30 seconds pass some processes still survive, they are
 sent the KILL signal.
 
-The killing is implemented using `IPC::Run`'s `kill_kill()`.
+The killing is implemented using <pm:IPC::Run>'s `kill_kill()`.
 
 Upon timeout, exit code is set to 124.
 
 _
+            tags => ['category:timeout'],
         },
         restart => {
             schema => [bool => default => 1],
             summary => 'If set to true, do restart',
+            tags => ['category:restart'],
         },
         # not yet defined
         #restart_delay => {
         #    schema => ['duration*', default=>0],
+        #    tags => ['category:restart'],
         #},
         #check_alive => {
         #    # not yet defined, can supply a custom coderef, or specify some
         #    # standard checks like TCP/UDP connection to some port, etc.
         #    schema => 'any*',
         #},
+        no_screensaver => {
+            summary => 'Prevent screensaver from being activated',
+            schema => ['bool*', is=>1],
+            tags => ['category:screensaver'],
+        },
     },
     args_rels => {
         'dep_all&' => [
@@ -264,6 +280,7 @@ sub govern_process {
     $self->{args} = \%args;
 
     require Proc::Killfam if $args{killfam};
+    require Screensaver::Any if $args{no_screensaver};
 
     my $debug = $ENV{DEBUG};
     $self->{debug} = $debug;
@@ -305,6 +322,8 @@ sub govern_process {
     my $lwhigh = $args{load_high_limit}  // 1.25;
     my $lwlow  = $args{load_low_limit}   // 0.25;
 
+    my $noss   = $args{no_screensaver};
+
     ###
 
     my $out;
@@ -385,6 +404,7 @@ sub govern_process {
     local $SIG{CHLD} = $chld_handler if $args{restart};
 
     my $lastlw_time;
+    my ($noss_screensaver, $noss_timeout, $noss_lastprevent_time);
 
   MAIN_LOOP:
     while (1) {
@@ -449,6 +469,41 @@ sub govern_process {
             $lastlw_time = $now;
         }
 
+      NOSS:
+        {
+            last unless $noss;
+            last unless !$noss_lastprevent_time ||
+                $noss_lastprevent_time <= ($now-$noss_timeout+5);
+            say "D:Preventing screensaver from activating" if $debug;
+            if (!$noss_lastprevent_time) {
+                $noss_screensaver = Screensaver::Any::detect_screensaver();
+                if (!$noss_screensaver) {
+                    warn "Can't detect any known screensaver, ".
+                        "will skip preventing screensaver from activating\n";
+                    $noss = 0;
+                    last NOSS;
+                }
+                my $res = Screensaver::Any::get_screensaver_timeout(
+                    screensaver => $noss_screensaver,
+                );
+                if ($res->[0] != 200) {
+                    warn "Can't get screensaver timeout ($res->[0]: $res->[1])".
+                        ", will skip preventing screensaver from activating\n";
+                    $noss = 0;
+                    last NOSS;
+                }
+                $noss_timeout = $res->[2];
+            }
+            my $res = Screensaver::Any::prevent_screensaver_activated(
+                screensaver => $noss_screensaver,
+            );
+            if ($res->[0] != 200) {
+                warn "Can't prevent screensaver from activating ".
+                    "($res->[0]: $res->[1])\n";
+            }
+            $noss_lastprevent_time = $now;
+        }
+
     } # MAINLOOP
 
   EXIT:
@@ -470,7 +525,7 @@ Proc::Govern - Run child process and govern its various aspects
 
 =head1 VERSION
 
-This document describes version 0.18 of Proc::Govern (from Perl distribution Proc-Govern), released on 2016-04-07.
+This document describes version 0.19 of Proc::Govern (from Perl distribution Proc-Govern), released on 2017-03-31.
 
 =head1 SYNOPSIS
 
@@ -585,12 +640,16 @@ Not yet tested on Win32.
 =head1 FUNCTIONS
 
 
-=head2 govern_process(%args) -> int
+=head2 govern_process
+
+Usage:
+
+ govern_process(%args) -> int
 
 Run child process and govern its various aspects.
 
-It basically uses C<IPC::Run> and a loop to check various conditions during the
-lifetime of the child process.
+It basically uses L<IPC::Run> and a loop to check various conditions during
+the lifetime of the child process.
 
 TODO: restart_delay, check_alive.
 
@@ -604,7 +663,7 @@ Arguments ('*' denotes required arguments):
 
 Command to run.
 
-Passed to C<IPC::Run>'s C<start()>.
+Passed to L<IPC::Run>'s C<start()>.
 
 =item * B<killfam> => I<bool>
 
@@ -613,7 +672,7 @@ Instead of kill, use killfam (kill family of process).
 This can be useful e.g. to control load more successfully, if the
 load-generating processes are the subchildren of the one we're governing.
 
-This requires C<Proc::Killfam> CPAN module, which is installed separately.
+This requires L<Proc::Killfam> CPAN module, which is installed separately.
 
 =item * B<load_check_every> => I<duration> (default: 10)
 
@@ -622,14 +681,14 @@ Frequency of load checking.
 =item * B<load_high_limit> => I<int|code>
 
 Limit above which program should be suspended, if load watching is enabled. If
-integer, will be compared against C<< Unix::Uptime-E<gt>load >>'s C<$load1> value.
+integer, will be compared against L<Unix::Uptime>C<< -E<gt>load >>'s C<$load1> value.
 Alternatively, you can provide a custom routine here, code should return true if
 load is considered too high.
 
 =item * B<load_low_limit> => I<int|code>
 
 Limit below which program should resume, if load watching is enabled. If
-integer, will be compared against C<< Unix::Uptime-E<gt>load >>'s C<$load1> value.
+integer, will be compared against L<Unix::Uptime>C<< -E<gt>load >>'s C<$load1> value.
 Alternatively, you can provide a custom routine here, code should return true if
 load is considered low.
 
@@ -642,13 +701,13 @@ too high and resumed if system load returns to a lower limit.
 
 Will be passed as arguments to `File::Write::Rotate`.
 
-Specify logging for STDERR. Logging will be done using C<File::Write::Rotate>.
+Specify logging for STDERR. Logging will be done using L<File::Write::Rotate>.
 Known hash keys: C<dir> (STR, defaults to C</var/log>, directory, preferably
 absolute, where the log file(s) will reside, should already exist and be
-writable, will be passed to C<File::Write::Rotate>'s constructor), C<size> (int,
-also passed to C<File::Write::Rotate>'s constructor), C<histories> (int, also
-passed to C<File::Write::Rotate>'s constructor), C<period> (str, also passed to
-C<File::Write::Rotate>'s constructor).
+writable, will be passed to L<File::Write::Rotate>'s constructor), C<size>
+(int, also passed to L<File::Write::Rotate>'s constructor), C<histories> (int,
+also passed to L<File::Write::Rotate>'s constructor), C<period> (str, also
+passed to L<File::Write::Rotate>'s constructor).
 
 =item * B<log_stdout> => I<hash>
 
@@ -657,10 +716,14 @@ Will be passed as arguments to `File::Write::Rotate`.
 =item * B<name> => I<str>
 
 Should match regex C<\A\w+\z>. Used in several places, e.g. passed as C<prefix> in
-C<File::Write::Rotate>'s constructor as well as used as name of PID file.
+L<File::Write::Rotate>'s constructor as well as used as name of PID file.
 
 If not given, will be taken from command.
 
+=item * B<no_screensaver> => I<bool>
+
+Prevent screensaver from being activated.
+
 =item * B<on_multiple_instance> => I<str>
 
 Can be set to C<exit> to silently exit when there is already a running instance.
@@ -686,7 +749,7 @@ Just like `show_stderr`, but for STDOUT.
 =item * B<single_instance> => I<bool> (default: 0)
 
 If set to true, will prevent running multiple instances simultaneously.
-Implemented using C<Proc::PID::File>. You will also normally have to set
+Implemented using L<Proc::PID::File>. You will also normally have to set
 C<pid_dir>, unless your script runs as root, in which case you can use the
 default C</var/run>.
 
@@ -698,7 +761,7 @@ After this time is reached, process (and all its descendants) are first sent the
 TERM signal. If after 30 seconds pass some processes still survive, they are
 sent the KILL signal.
 
-The killing is implemented using C<IPC::Run>'s C<kill_kill()>.
+The killing is implemented using L<IPC::Run>'s C<kill_kill()>.
 
 Upon timeout, exit code is set to 124.
 
@@ -806,7 +869,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2016 by perlancar@cpan.org.
+This software is copyright (c) 2017, 2016, 2015, 2014, 2013, 2012 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
